@@ -13,11 +13,14 @@ import cv2  # 这个库也要打包，所以import进来
 from PyQt5 import QtWidgets, QtCore
 
 from xllabelme import __appname__, __version__
+from xllabelme.app import MainWindow
 from xllabelme.xlapp import XlMainWindow
 from xllabelme.config import get_config
 from xllabelme.logger import logger
 from xllabelme.utils import newIcon
 
+from pyxllib.prog.pupil import dprint  # 在任意地方均可以使用dprint调试
+
 
 def default_argument_parser():
     parser = argparse.ArgumentParser()
@@ -58,7 +61,7 @@ def default_argument_parser():
         dest="store_data",
         action="store_false",
         help="stop storing image data to JSON file",
-        default=True,
+        default=False,
     )
     parser.add_argument(
         "--autosave",
@@ -113,6 +116,12 @@ def default_argument_parser():
         help="epsilon to find nearest vertex on canvas",
         default=argparse.SUPPRESS,
     )
+    parser.add_argument(
+        "--lang",
+        type=str,
+        help="set language",
+        default=QtCore.QLocale.system().name(),
+    )
     return parser
 
 
@@ -173,12 +182,13 @@ def main(mainwin=XlMainWindow):
 
     translator = QtCore.QTranslator()
     translator.load(
-        QtCore.QLocale.system().name(),
+        args.lang,  # 写不存在的值也没影响，默认就是变成英文
         osp.dirname(osp.abspath(__file__)) + "/translate",
     )
+    # translator.load('translate/zh_CN.qm')  # 如果只要英文的，把这句注释掉就行
     app = QtWidgets.QApplication(sys.argv)
     app.setApplicationName(__appname__)
-    app.setWindowIcon(newIcon("icon"))
+    app.setWindowIcon(newIcon("icon.png"))
     app.installTranslator(translator)
     win = mainwin(
         config=config,
@@ -186,7 +196,6 @@ def main(mainwin=XlMainWindow):
         output_file=output_file,
         output_dir=output_dir,
     )
-
     if reset_config:
         logger.info("Resetting Qt config: %s" % win.settings.fileName())
         win.settings.clear()
