@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import functools
+import html
 import math
 import os
 import os.path as osp
@@ -10,7 +11,7 @@ import webbrowser
 import imgviz
 import natsort
 from qtpy import QtCore
-from qtpy.QtCore import Qt
+from qtpy.QtCore import Qt, QPointF
 from qtpy import QtGui
 from qtpy import QtWidgets
 
@@ -26,16 +27,17 @@ from xllabelme.shape import Shape
 from xllabelme.widgets import BrightnessContrastDialog
 from xllabelme.widgets import Canvas
 from xllabelme.widgets import FileDialogPreview
-from xllabelme.widgets import DefaultLabelDialog
+# from xllabelme.widgets import DefaultLabelDialog
+from xllabelme.widgets import LabelDialogExt
 from xllabelme.widgets import LabelListWidget
 from xllabelme.widgets import LabelListWidgetItem
 from xllabelme.widgets import ToolBar
 from xllabelme.widgets import UniqueLabelQListWidget
 from xllabelme.widgets import ZoomWidget
+from xllabelme.utils.qt import newIcon
 
 from pyxllib.file.specialist import XlPath
 
-
 # FIXME
 # - [medium] Set max zoom value to something big enough for FitWidth/Window
 
@@ -59,6 +61,9 @@ def set_default_shape_colors(cfg):
         *cfg["shape"]["hvertex_fill_color"]
     )
 
+    # Set point size from config file
+    Shape.point_size = cfg["shape"]["point_size"]
+
 
 class MainWindow(QtWidgets.QMainWindow):
     LABEL_COLORMAP = imgviz.label_colormap(value=200)
@@ -94,8 +99,9 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self._noSelectionSlot = False
 
+        self._copied_shapes = None
         # Main widgets and related state.
-        self.labelDialog = DefaultLabelDialog(
+        self.labelDialog = LabelDialogExt(
             parent=self,
             labels=self._config["labels"],
             sort_labels=self._config["sort_labels"],
@@ -167,9 +173,11 @@ class MainWindow(QtWidgets.QMainWindow):
         self.setAcceptDrops(True)
 
         self.canvas = self.labelList.canvas = Canvas(
+            self,
             epsilon=self._config["epsilon"],
             double_click=self._config["canvas"]["double_click"],
             num_backups=self._config["canvas"]["num_backups"],
+            crosshair=self._config["canvas"]["crosshair"],
         )
         self.canvas.zoomRequest.connect(self.zoomRequest)
 
@@ -200,9 +208,11 @@ class MainWindow(QtWidgets.QMainWindow):
             getattr(self, dock).setFeatures(features)
             if self._config[dock]["show"] is False:
                 getattr(self, dock).setVisible(False)
+                # getattr(self, dock).setFeatures(QtWidgets.QDockWidget.NoDockWidgetFeatures)
 
-        self.addDockWidget(Qt.RightDockWidgetArea, self.flag_dock)
-        self.addDockWidget(Qt.RightDockWidgetArea, self.label_dock)
+        # 标记、标签列表 两个窗口默认不展示
+        # self.addDockWidget(Qt.RightDockWidgetArea, self.flag_dock)
+        # self.addDockWidget(Qt.RightDockWidgetArea, self.label_dock)
         self.addDockWidget(Qt.RightDockWidgetArea, self.shape_dock)
         self.addDockWidget(Qt.RightDockWidgetArea, self.file_dock)
 
@@ -291,19 +301,19 @@ class MainWindow(QtWidgets.QMainWindow):
         saveAuto.setChecked(self._config["auto_save"])
 
         saveWithImageData = action(
-            text="Save With Image Data",
+            text=self.tr("Save With Image Data"),
             slot=self.enableSaveImageWithData,
-            tip="Save image data in label file",
+            tip=self.tr("Save image data in label file"),
             checkable=True,
             checked=False,
         )
 
         close = action(
-            "&Close",
+            self.tr("&Close"),
             self.closeFile,
             shortcuts["close"],
             "close",
-            "Close current file",
+            self.tr("Close current file"),
         )
 
         toggle_keep_prev_mode = action(
@@ -389,6 +399,7 @@ class MainWindow(QtWidgets.QMainWindow):
             self.tr("Create a duplicate of the selected polygons"),
             enabled=False,
         )
+        # 有 duplicate 其实就不需要这个重复的复制粘贴了。操作上还需要，但是菜单里不需要显示。
         copy = action(
             self.tr("Copy Polygons"),
             self.copySelectedShape,
@@ -398,7 +409,7 @@ class MainWindow(QtWidgets.QMainWindow):
             enabled=False,
         )
         paste = action(
-            self.tr("Paste Polygons"),
+            self.tr("Paste polygons"),
             self.pasteSelectedShape,
             shortcuts["paste_polygon"],
             "paste",
@@ -414,11 +425,11 @@ class MainWindow(QtWidgets.QMainWindow):
             enabled=False,
         )
         removePoint = action(
-            text="Remove Selected Point",
+            text=self.tr("Remove Selected Point"),
             slot=self.removeSelectedPoint,
             shortcut=shortcuts["remove_selected_point"],
             icon="edit",
-            tip="Remove selected point from polygon",
+            tip=self.tr("Remove selected point from polygon"),
             enabled=False,
         )
 
@@ -521,11 +532,11 @@ class MainWindow(QtWidgets.QMainWindow):
             enabled=False,
         )
         brightnessContrast = action(
-            "&Brightness Contrast",
+            self.tr("&Brightness Contrast"),
             self.brightnessContrast,
             None,
             "color",
-            "Adjust brightness and contrast",
+            self.tr("Adjust brightness and contrast"),
             enabled=False,
         )
         # Group zoom controls into a list for easier toggling.
@@ -637,8 +648,8 @@ class MainWindow(QtWidgets.QMainWindow):
                 editMode,
                 edit,
                 duplicate,
-                copy,
-                paste,
+                # copy,
+                # paste,
                 delete,
                 undo,
                 undoLastPoint,
@@ -664,6 +675,8 @@ class MainWindow(QtWidgets.QMainWindow):
             file=self.menu(self.tr("&File")),
             edit=self.menu(self.tr("&Edit")),
             view=self.menu(self.tr("&View")),
+            settings=self.menu(self.tr('&Settings')),
+            # label=self.menu(self.tr("&Label")),  # 标注文本相关工具
             help=self.menu(self.tr("&Help")),
             recentFiles=QtWidgets.QMenu(self.tr("Open &Recent")),
             labelList=labelMenu,
@@ -729,7 +742,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.tools = self.toolbar("Tools")
         # Menu buttons on Left
         self.actions.tool = (
-            open_,
+            # open_,
             opendir,
             openNextImg,
             openPrevImg,
@@ -739,8 +752,8 @@ class MainWindow(QtWidgets.QMainWindow):
             createMode,
             editMode,
             duplicate,
-            copy,
-            paste,
+            # copy,
+            # paste,
             delete,
             undo,
             brightnessContrast,
@@ -749,7 +762,7 @@ class MainWindow(QtWidgets.QMainWindow):
             fitWidth,
         )
 
-        self.statusBar().showMessage(str(self.tr("%s started.")) % __appname__)
+        self.statusBar().showMessage(self.tr("%s started.") % __appname__)
         self.statusBar().show()
 
         if output_file is not None and self._config["auto_save"]:
@@ -787,8 +800,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         # XXX: Could be completely declarative.
         # Restore application settings.
-        self.settings = QtCore.QSettings("labelme", "labelme")
-        # FIXME: QSettings.value can return None on PyQt4
+        self.settings = QtCore.QSettings("labelme", "labelme")  # 这个配置好像是写到注册表中的~
         self.recentFiles = self.settings.value("recentFiles", []) or []
         size = self.settings.value("window/size", QtCore.QSize(600, 500))
         position = self.settings.value("window/position", QtCore.QPoint(0, 0))
@@ -852,7 +864,7 @@ class MainWindow(QtWidgets.QMainWindow):
             self.actions.createLineStripMode,
             self.actions.editMode,
         )
-        utils.addActions(self.menus.edit, actions + self.actions.editMenu)
+        utils.addActions(self.menus.edit, list(actions) + list(self.actions.editMenu))
 
     def setDirty(self):
         # Even if we autosave the file, we keep the ability to undo
@@ -1076,17 +1088,18 @@ class MainWindow(QtWidgets.QMainWindow):
         if shape.group_id is None:
             item.setText(
                 '{} <font color="#{:02x}{:02x}{:02x}">●</font>'.format(
-                    shape.label, *shape.fill_color.getRgb()[:3]
+                    html.escape(shape.label), *shape.fill_color.getRgb()[:3]
                 )
             )
         else:
             item.setText("{} ({})".format(shape.label, shape.group_id))
 
         self.setDirty()
-        if not self.uniqLabelList.findItemsByLabel(shape.label):
-            item = QtWidgets.QListWidgetItem()
-            item.setData(Qt.UserRole, shape.label)
+        if self.uniqLabelList.findItemByLabel(shape.label) is None:
+            item = self.uniqLabelList.createItemFromLabel(shape.label)
             self.uniqLabelList.addItem(item)
+            rgb = self._get_rgb_by_label(shape.label)
+            self.uniqLabelList.setItemLabel(item, shape.label, rgb)
 
     def fileSearchChanged(self):
         self.importDirImages(
@@ -1138,7 +1151,7 @@ class MainWindow(QtWidgets.QMainWindow):
             text = "{} ({})".format(shape.label, shape.group_id)
         label_list_item = LabelListWidgetItem(text, shape)
         self.labelList.addItem(label_list_item)
-        if not self.uniqLabelList.findItemsByLabel(shape.label):
+        if self.uniqLabelList.findItemByLabel(shape.label) is None:
             item = self.uniqLabelList.createItemFromLabel(shape.label)
             self.uniqLabelList.addItem(item)
             rgb = self._get_rgb_by_label(shape.label)
@@ -1150,7 +1163,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self._update_shape_color(shape)
         label_list_item.setText(
             '{} <font color="#{:02x}{:02x}{:02x}">●</font>'.format(
-                text, *shape.fill_color.getRgb()[:3]
+                html.escape(text), *shape.fill_color.getRgb()[:3]
             )
         )
 
@@ -1165,10 +1178,15 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def _get_rgb_by_label(self, label):
         if self._config["shape_color"] == "auto":
-            item = self.uniqLabelList.findItemsByLabel(label)[0]
+            item = self.uniqLabelList.findItemByLabel(label)
+            if item is None:
+                item = self.uniqLabelList.createItemFromLabel(label)
+                self.uniqLabelList.addItem(item)
+                rgb = self._get_rgb_by_label(label)
+                self.uniqLabelList.setItemLabel(item, label, rgb)
             label_id = self.uniqLabelList.indexFromItem(item).row() + 1
             label_id += self._config["shift_auto_shape_color"]
-            return LABEL_COLORMAP[label_id % len(LABEL_COLORMAP)]
+            return self.LABEL_COLORMAP[label_id % len(self.LABEL_COLORMAP)]
         elif (
                 self._config["shape_color"] == "manual"
                 and self._config["label_colors"]
@@ -1294,7 +1312,9 @@ class MainWindow(QtWidgets.QMainWindow):
     def duplicateSelectedShape(self):
         added_shapes = self.canvas.duplicateSelectedShapes()
         self.labelList.clearSelection()
+        # dx, dy = 0, 0
         for shape in added_shapes:
+            # shape.points = [QPointF(p.x() + dx, p.y() + dy) for p in shape.points]
             self.addLabel(shape)
         self.setDirty()
 
@@ -1329,51 +1349,7 @@ class MainWindow(QtWidgets.QMainWindow):
     # Callback functions:
 
     def newShape(self):
-        """Pop-up and give focus to the label editor.
-
-        position MUST be in global coordinates.
-        """
-        items = self.uniqLabelList.selectedItems()
-        text = None
-        if items:
-            text = items[0].data(Qt.UserRole)
-        flags = {}
-        group_id = None
-        if self._config["display_label_popup"] or not text:
-            previous_text = self.labelDialog.edit.text()
-            # if self.xllabel.cfg['autodict']:  # ckz
-            if True:  # 使用我的xllabelme，会把数据全部都强制升为字典，不再考虑兼容label原版格式，这样很多扩展功能开发会简洁的多
-                shape = Shape()
-                shape.label = self.xllabel.get_default_label(shape=self.canvas.shapes[-1])
-                shape = self.labelDialog.popUp2(shape, self)
-                if shape is not None:
-                    text, flags, group_id = shape.label, shape.flags, shape.group_id
-            else:
-                text, flags, group_id = self.labelDialog.popUp(text)
-
-            if not text:
-                self.labelDialog.edit.setText(previous_text)
-
-        if text and not self.validateLabel(text):
-            self.errorMessage(
-                self.tr("Invalid label"),
-                self.tr("Invalid label '{}' with validation type '{}'").format(
-                    text, self._config["validate_label"]
-                ),
-            )
-            text = ""
-        if text:
-            self.labelList.clearSelection()
-            shape = self.canvas.setLastLabel(text, flags)
-            shape.group_id = group_id
-            self.addLabel(shape)
-            self.actions.editMode.setEnabled(True)
-            self.actions.undoLastPoint.setEnabled(False)
-            self.actions.undo.setEnabled(True)
-            self.setDirty()
-        else:
-            self.canvas.undoLastLine()
-            self.canvas.shapesBackups.pop()
+        self.project.new_shape()
 
     def scrollRequest(self, delta, orientation):
         units = -delta * 0.1  # natural scroll
@@ -1382,7 +1358,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.setScroll(orientation, value)
 
     def setScroll(self, orientation, value):
-        self.scrollBars[orientation].setValue(value)
+        self.scrollBars[orientation].setValue(int(value))
         self.scroll_values[orientation][self.filename] = value
 
     def setZoom(self, value):
@@ -1947,10 +1923,10 @@ class MainWindow(QtWidgets.QMainWindow):
         if not self.canvas.hShape.points:
             self.canvas.deleteShape(self.canvas.hShape)
             self.remLabels([self.canvas.hShape])
-            self.setDirty()
             if self.noShapes():
                 for action in self.actions.onShapesPresent:
                     action.setEnabled(False)
+        self.setDirty()
 
     def deleteSelectedShape(self):
         yes, no = QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No
@@ -1959,7 +1935,8 @@ class MainWindow(QtWidgets.QMainWindow):
             "proceed anyway?"
         ).format(len(self.canvas.selectedShapes))
         if not self.delete_selected_shape_with_warning_action.isChecked() or \
-                yes == QtWidgets.QMessageBox.warning(self, self.tr("Attention"), msg, yes | no, yes):
+                yes == QtWidgets.QMessageBox.warning(self, self.tr("Attention"), msg, yes | no, yes
+        ):
             self.remLabels(self.canvas.deleteSelected())
             self.setDirty()
             if self.noShapes():
@@ -1993,7 +1970,7 @@ class MainWindow(QtWidgets.QMainWindow):
             QtWidgets.QFileDialog.getExistingDirectory(
                 self,
                 self.tr("%s - Open Directory") % __appname__,
-                defaultOpenDirPath,
+                str(defaultOpenDirPath),
                 QtWidgets.QFileDialog.ShowDirsOnly
                 | QtWidgets.QFileDialog.DontResolveSymlinks,
             )
