@@ -14,7 +14,9 @@ here = osp.dirname(osp.abspath(__file__))
 
 def newIcon(icon):
     icons_dir = osp.join(here, "../icons")
-    return QtGui.QIcon(osp.join(":/", icons_dir, "%s.png" % icon))
+    if osp.splitext(icon)[-1] != '.png':
+        icon += '.png'
+    return QtGui.QIcon(osp.join(":/", icons_dir, icon))
 
 
 def newButton(text, icon=None, slot=None):
